""""Base classes and mixins for AbiPy panels."""

#import abc
import param
import panel as pn
import panel.widgets as pnw
import bokeh.models.widgets as bkw

from monty.functools import lazy_property


def gen_id(n=1, pre="uuid-"):
    """
    Generate ``n`` universally unique identifiers prepended with ``pre`` string.
    Return string if n == 1 or list of strings if n > 1
    """
    # The HTML4 spec says:
    # ID and NAME tokens must begin with a letter ([A-Za-z]) and may be followed by any number of letters,
    # digits ([0-9]), hyphens ("-"), underscores ("_"), colons (":"), and periods (".").
    import uuid
    if n == 1:
        return pre + str(uuid.uuid4())
    elif n > 1:
        return [pre + str(uuid.uuid4()) for i in range(n)]
    else:
        raise ValueError("n must be > 0 but got %s" % str(n))


def sizing_mode_select(name="sizing_mode", value="scale_both"):
    """
    Widget to select the value of sizing_mode. See https://panel.holoviz.org/user_guide/Customization.html
    """
    return pn.widgets.Select(name=name, value=value, options=["fixed",
                 "stretch_width", "stretch_height", "stretch_both",
                 "scale_height", "scale_width", "scale_both"])


class AbipyParameterized(param.Parameterized):

    verbose = param.Integer(0, bounds=(0, None), doc="Verbosity Level")
    mpi_procs = param.Integer(1, bounds=(1, None), doc="Number of MPI processes used when runnin Fortran code")
    #fontsize =

    #def get_global_widgets(self):
    #    return pn.Column(self.verbose, self.mpi_procs)

    @lazy_property
    def fig_kwargs(self):
        """Default arguments passed to AbiPy plot methods."""
        return dict(show=False, fig_close=True)

    @staticmethod
    def _mp(fig):
        return pn.pane.Matplotlib(fig, sizing_mode="scale_width")

    @staticmethod
    def _df(df, disabled=True, sizing_mode="scale_width"):
        return pn.widgets.DataFrame(df, disabled=disabled, sizing_mode=sizing_mode)


#class PanelWithNcFile(AbipyParameterized): #, metaclass=abc.ABCMeta):
#    """
#    This frame allows the user to inspect the dimensions and the variables reported in  a netcdf file.
#    Tab showing information on the netcdf file.
#    """


class PanelWithElectronBands(AbipyParameterized): #, metaclass=abc.ABCMeta):

    # Bands plot
    with_gaps = pnw.Checkbox(name='Show gaps')
    #ebands_ylims
    #ebands_e0
    # e0: Option used to define the zero of energy in the band structure plot. Possible values:
    #     - `fermie`: shift all eigenvalues to have zero energy at the Fermi energy (`self.fermie`).
    #     -  Number e.g e0=0.5: shift all eigenvalues to have zero energy at 0.5 eV
    #     -  None: Don't shift energies, equivalent to e0=0
    set_fermie_to_vbm = pnw.Checkbox(name="Set Fermie to VBM")

    plot_ebands_btn = pnw.Button(name="Plot e-bands", button_type='primary')

    # DOS plot.
    edos_method = pnw.Select(name="e-DOS method", options=["gaussian", "tetra"])
    edos_step = pnw.Spinner(name='e-DOS step (eV)', value=0.1, step=0.05, start=1e-6, end=None)
    edos_width = pnw.Spinner(name='e-DOS Gaussian broadening (eV)', value=0.2, step=0.05, start=1e-6, end=None)
    plot_edos_btn = pnw.Button(name="Plot e-DOS", button_type='primary')

    # Fermi surface plot.
    fs_viewer = pnw.Select(name="FS viewer", options=["matplotlib", "xcrysden"])
    plot_fermi_surface_btn = pnw.Button(name="Plot Fermi surface", button_type='primary')

    #@abc.abstractproperty
    #def ebands(self):
    #    """Returns the |ElectronBands| object."""

    def get_plot_ebands_widgets(self):
        """Widgets to plot ebands."""
        return pn.Column(self.with_gaps, self.set_fermie_to_vbm, self.plot_ebands_btn)

    @param.depends('plot_ebands_btn.clicks')
    def on_plot_ebands_btn(self):
        """Button triggering ebands plot."""
        if self.plot_ebands_btn.clicks == 0: return
        if self.set_fermie_to_vbm.value:
            self.ebands.set_fermie_to_vbm()

        fig1 = self.ebands.plot(e0="fermie", ylims=None,
            with_gaps=self.with_gaps.value, max_phfreq=None, fontsize=8, **self.fig_kwargs)

        fig2 = self.ebands.kpoints.plot(**self.fig_kwargs)
        row = pn.Row(self._mp(fig1), self._mp(fig2)) #, sizing_mode='scale_width')
        text = bkw.PreText(text=self.ebands.to_string(verbose=self.verbose))
        return pn.Column(row, text, sizing_mode='scale_width')

    def get_plot_edos_widgets(self):
        """Widgets to compute e-DOS."""
        return pn.Column(self.edos_method, self.edos_step, self.edos_width, self.plot_edos_btn)

    @param.depends('plot_edos_btn.clicks')
    def on_plot_edos_btn(self):
        """Button triggering edos plot."""
        if self.plot_edos_btn.clicks == 0: return
        edos = self.ebands.get_edos(method=self.edos_method.value, step=self.edos_step.value, width=self.edos_width.value)
        fig = edos.plot(**self.fig_kwargs)
        return pn.Row(self._mp(fig), sizing_mode='scale_width')

    def get_plot_fermi_surface_widgets(self):
        """Widgets to compute e-DOS."""
        return pn.Column(self.fs_viewer, self.plot_fermi_surface_btn)

    @param.depends('plot_fermi_surface_btn.clicks')
    def on_plot_fermi_surface_btn(self):
        if self.plot_fermi_surface_btn.clicks == 0: return

        # Cache eb3d
        if hasattr(self, "_eb3d"):
            eb3d = self._eb3d
        else:
            # Build ebands in full BZ.
            eb3d = self._eb3d = self.ebands.get_ebands3d()

        if self.fs_viewer.value == "matplotlib":
            # Use matplotlib to plot isosurfaces corresponding to the Fermi level (default)
            # Warning: requires skimage package, rendering could be slow.
            fig = eb3d.plot_isosurfaces(e0="fermie", cmap=None, **self.fig_kwargs)
            return pn.Row(self._mp(fig), sizing_mode='scale_width')

        else:
            raise ValueError("Invalid choice: %s" % self.fs_viewer.value)

        #elif self.fs_viewer.value == "xcrysden":
            # Alternatively, it's possible to export the data in xcrysden format
            # and then use `xcrysden --bxsf mgb2.bxsf`
            #eb3d.to_bxsf("mgb2.bxsf")
            # If you have mayavi installed, try:
            #eb3d.mvplot_isosurfaces()


class BaseRobotPanel(AbipyParameterized):
    """pass"""


class PanelWithEbandsRobot(BaseRobotPanel): #, metaclass=abc.ABCMeta):
    """
    Mixin class for panels with a robot that owns a list of of |ElectronBands|
    """

    # Widgets to plot ebands.
    ebands_plotter_mode = pnw.Select(name="Plot Mode", value="gridplot",
        options=["gridplot", "combiplot", "boxplot", "combiboxplot"]) # "animate",
    ebands_plotter_btn = pnw.Button(name="Plot", button_type='primary')
    ebands_df_checkbox = pnw.Checkbox(name='With Ebands DataFrame', value=False)

    # Widgets to plot edos.
    edos_plotter_mode = pnw.Select(name="Plot Mode", value="gridplot",
        options=["gridplot", "combiplot"])
    edos_plotter_btn = pnw.Button(name="Plot", button_type='primary')

    def get_ebands_plotter_widgets(self):
        return pn.Column(self.ebands_plotter_mode, self.ebands_df_checkbox, self.ebands_plotter_btn)

    @param.depends("ebands_plotter_btn.clicks")
    def on_ebands_plotter_btn(self):
        if self.ebands_plotter_btn.clicks == 0: return
        ebands_plotter = self.robot.get_ebands_plotter()
        plot_mode = self.ebands_plotter_mode.value
        plotfunc = getattr(ebands_plotter, plot_mode, None)
        if plotfunc is None:
            raise ValueError("Don't know how to handle plot_mode: %s" % plot_mode)

        fig = plotfunc(**self.fig_kwargs)
        col = pn.Column(self._mp(fig), sizing_mode='scale_width')
        if self.ebands_df_checkbox.value:
            df = ebands_plotter.get_ebands_frame(with_spglib=True)
            col.append(self._df(df))

        return pn.Row(col, sizing_mode='scale_width')

    def get_edos_plotter_widgets(self):
        return pn.Column(self.edos_plotter_mode, self.edos_plotter_btn)

    @param.depends("edos_plotter_btn.clicks")
    def on_edos_plotter_btn(self):
        if self.edos_plotter_btn.clicks == 0: return
        edos_plotter = self.robot.get_edos_plotter()
        plot_mode = self.edos_plotter_mode.value
        plotfunc = getattr(edos_plotter, plot_mode, None)
        if plotfunc is None:
            raise ValueError("Don't know how to handle plot_mode: %s" % plot_mode)

        fig = plotfunc(**self.fig_kwargs)
        return pn.Row(pn.Column(self._mp(fig)), sizing_mode='scale_width')



""""Panels for AbiPy flows."""
import param
import panel as pn
import panel.widgets as pnw
import bokeh.models.widgets as bkw

from io import StringIO
from abipy.panels.core import AbipyParameterized


class FlowPanel(AbipyParameterized):
    """
    """
    verbose = pn.widgets.IntSlider(start=0, end=10, step=1, value=0)

    engine = pn.widgets.Select(value="fdp",
        options=['dot', 'neato', 'twopi', 'circo', 'fdp', 'sfdp', 'patchwork', 'osage'])
    dirtree = pn.widgets.Checkbox(name='Dirtree', value=False)
    graphviz_btn = pn.widgets.Button(name="Show graph", button_type='primary')

    status_btn = pn.widgets.Button(name="Show status", button_type='primary')
    history_btn = pn.widgets.Button(name="Show history", button_type='primary')
    debug_btn = pn.widgets.Button(name="Debug", button_type='primary')
    events_btn = pn.widgets.Button(name="Events", button_type='primary')
    corrections_btn = pn.widgets.Button(name="Corrections", button_type='primary')
    handlers_btn = pn.widgets.Button(name="Handlers", button_type='primary')

    vars_text = pn.widgets.TextInput(name='Abivars', placeholder='Enter list of variables separated by comma')
    vars_btn = pn.widgets.Button(name="Show Variables", button_type='primary')

    dims_btn = pn.widgets.Button(name="Show Dimensions", button_type='primary')

    structures_btn = pn.widgets.Button(name="Show Structures", button_type='primary')
    structures_io_checkbox = pn.widgets.CheckBoxGroup(
        name='Input/Output Structure', value=['output'], options=['input', 'output'], inline=True)

    # Widgets to plot ebands.
    ebands_btn = pn.widgets.Button(name="Show Ebands", button_type='primary')
    ebands_plotter_mode = pnw.Select(name="Plot Mode", value="gridplot",
        options=["gridplot", "combiplot", "boxplot", "combiboxplot"]) # "animate",
    ebands_plotter_btn = pnw.Button(name="Plot", button_type='primary')
    ebands_df_checkbox = pnw.Checkbox(name='With Ebands DataFrame', value=False)
    ebands_ksamp_checkbox = pn.widgets.CheckBoxGroup(
        name='Input/Output Structure', value=["with_path", "with_ibz"], options=['with_path', 'with_ibz'], inline=True)

    #TODO: Implement widget for selected_nids(flow, options),
    #radio_group = pn.widgets.RadioButtonGroup(
    #   name='Radio Button Group', options=['Biology', 'Chemistry', 'Physics'], button_type='success')

    def __init__(self, flow, **params):
        super().__init__(**params)
        self.flow = flow

    @param.depends('status_btn.clicks')
    def on_status_btn(self):
        if self.status_btn.clicks == 0: return
        stream = StringIO()
        self.flow.show_status(stream=stream, verbose=self.verbose.value)
        return pn.Row(bkw.PreText(text=stream.getvalue()))

    @param.depends('history_btn.clicks')
    def on_history_btn(self):
        if self.history_btn.clicks == 0: return
        stream = StringIO()
        #flow.show_history(status=options.task_status, nids=selected_nids(flow, options),
        #                  full_history=options.full_history, metadata=options.metadata)
        self.flow.show_history(stream=stream)
        return pn.Row(bkw.PreText(text=stream.getvalue()))

    @param.depends('graphviz_btn.clicks')
    def on_graphviz_btn(self):
        """
        """
        if self.graphviz_btn.clicks == 0: return
        node = self.flow
        if self.dirtree.value:
            graph = node.get_graphviz_dirtree(engine=self.engine.value)
        else:
            graph = node.get_graphviz(engine=self.engine.value)
        return pn.Column(graph)

    @param.depends('debug_btn.clicks')
    def on_debug_btn(self):
        if self.debug_btn.clicks == 0: return
        #TODO https://github.com/ralphbean/ansi2html ?
        stream = StringIO()
        #flow.debug(status=options.task_status, nids=selected_nids(flow, options))
        self.flow.debug(stream=stream)
        return pn.Row(bkw.PreText(text=stream.getvalue()))

    @param.depends('events_btn.clicks')
    def on_events_btn(self):
        if self.events_btn.clicks == 0: return
        stream = StringIO()
        self.flow.show_events(stream=stream)
        #flow.show_events(status=options.task_status, nids=selected_nids(flow, options))
        return pn.Row(bkw.PreText(text=stream.getvalue()))

    @param.depends('corrections_btn.clicks')
    def on_corrections_btn(self):
        if self.corrections_btn.clicks == 0: return
        stream = StringIO()
        self.flow.show_corrections(stream=stream)
        #flow.show_corrections(status=options.task_status, nids=selected_nids(flow, options))
        return pn.Row(bkw.PreText(text=stream.getvalue()))

    @param.depends('handlers_btn.clicks')
    def on_handlers_btn(self):
        #if self.handlers_btn.clicks == 0: return
        stream = StringIO()
        #if options.doc:
        #    flowtk.autodoc_event_handlers()
        #else:
        #show_events(self, status=None, nids=None, stream=sys.stdout):
        self.flow.show_event_handlers(verbose=self.verbose.value, stream=stream)
        return pn.Row(bkw.PreText(text=stream.getvalue()))

    @param.depends('vars_btn.clicks')
    def on_vars_btn(self):
        if self.vars_btn.clicks == 0: return
        if not self.vars_text.value: return
        varnames = [s.strip() for s in self.vars_text.value.split(",")]
        df = self.flow.compare_abivars(varnames=varnames, # nids=selected_nids(flow, options),
                                       printout=False, with_colors=False)
        return pn.Row(self._df(df))

    @param.depends('dims_btn.clicks')
    def on_dims_btn(self):
        if self.dims_btn.clicks == 0: return
        df = self.flow.get_dims_dataframe(# nids=selected_nids(flow, options),
                                          printout=False, with_colors=False)
        return pn.Row(self._df(df), sizing_mode="scale_width")

    @param.depends('structures_btn.clicks')
    def on_structures_btn(self):
        if self.structures_btn.clicks == 0: return
        what = ""
        if "input" in self.structures_io_checkbox.value: what += "i"
        if "output" in self.structures_io_checkbox.value: what += "o"
        dfs = self.flow.compare_structures(nids=None, # select_nids(flow, options),
                                           what=what,
                                           verbose=self.verbose.value, with_spglib=False, printout=False,
                                           with_colors=False)

        return pn.Row(self._df(dfs.lattice), sizing_mode="scale_width")

    @param.depends('ebands_plotter_btn.clicks')
    def on_ebands_btn(self):
        if self.ebands_plotter_btn.clicks == 0: return

        df, ebands_plotter = self.flow.compare_ebands(
                                nids=None, # select_nids(flow, options),
                                with_path="with_path" in self.ebands_ksamp_checkbox.value,
                                with_ibz="with_ibz" in self.ebands_ksamp_checkbox.value,
                                verbose=self.verbose.value,
                                with_spglib=False
                                )

        if ebands_plotter is None:
            return

        plot_mode = self.ebands_plotter_mode.value
        plotfunc = getattr(ebands_plotter, plot_mode, None)
        if plotfunc is None:
            raise ValueError("Don't know how to handle plot_mode: %s" % plot_mode)

        fig = plotfunc(**self.fig_kwargs)
        col = pn.Column(self._mp(fig))
        if self.ebands_df_checkbox.value:
            col.append(self._df(df))

        return pn.Row(col) #, sizing_mode='scale_width')

    def get_panel(self):
        """Return tabs with widgets to interact with the flow."""
        tabs = pn.Tabs(); app = tabs.append
        #row = pn.Row(bkw.PreText(text=self.ddb.to_string(verbose=self.verbose.value), sizing_mode="scale_both"))
        app(("Status", pn.Row(self.status_btn, self.on_status_btn)))
        app(("History", pn.Row(self.history_btn, self.on_history_btn)))
        app(("Events", pn.Row(self.events_btn, self.on_events_btn)))
        app(("Corrections", pn.Row(self.corrections_btn, self.on_corrections_btn)))
        app(("Handlers", pn.Row(self.handlers_btn, self.on_handlers_btn)))
        app(("Structures", pn.Row(pn.Column(self.structures_io_checkbox, self.structures_btn), self.on_structures_btn)))
        ws = pn.Column(self.ebands_plotter_mode, self.ebands_ksamp_checkbox, self.ebands_df_checkbox, self.ebands_plotter_btn)
        app(("Ebands", pn.Row(ws, self.on_ebands_btn)))
        app(("Abivars", pn.Row(pn.Column(self.vars_text, self.vars_btn), self.on_vars_btn)))
        app(("Dims", pn.Row(pn.Column(self.dims_btn), self.on_dims_btn)))
        app(("Debug", pn.Row(self.debug_btn, self.on_debug_btn)))
        app(("Graphviz", pn.Row(pn.Column(self.engine, self.dirtree, self.graphviz_btn),
                                self.on_graphviz_btn)))
        return tabs



#import os

__all__ = []

#_MODS = [
#    "buttons.py",
#    "core",
#    "apps",
#    "frames",
#    "grids",
#    "dialogs",
#    "panels",
#    "eggs",
#    "func1dframe",
#    "threads",
#    "utils",
#]

#_MODS = [os.path.join(os.path.dirname(__file__), f) for f in _MODS]

#for _mod in _MODS:
#    try:
#        exec('from .' + _mod + ' import *')
#        exec('__all__.extend(' + _mod + '.__all__)')
#        exec('del ' + _mod)
#    except ImportError:
#        pass

from abipy.gui.awx.buttons import *
from abipy.gui.awx.core import *
from abipy.gui.awx.apps import *
from abipy.gui.awx.frames import *
from abipy.gui.awx.grids import *
from abipy.gui.awx.dialogs import *
from abipy.gui.awx.panels import *
from abipy.gui.awx.eggs import *
from abipy.gui.awx.func1dframe import *
from abipy.gui.awx.threads import *
from abipy.gui.awx.utils import *



#!/usr/bin/env python
r"""
Multiple phonon bands
=========================

This example shows how to plot several phonon band structures on a grid.

We use two files produced by anaddb:

    trf2_5.out_PHBST.nc: phonon frequencies on a q-path in the BZ (used to plot the band dispersion)
    trf2_5.out_PHDOS.nc: phonon DOS compute with anaddb.
"""
from abipy import abilab
import abipy.data as abidata

import os
paths = [
    #"mgb2_444k_0.01tsmear_DDB",
    #"mgb2_444k_0.02tsmear_DDB",
    #"mgb2_444k_0.04tsmear_DDB",
    "mgb2_888k_0.01tsmear_DDB",
    #"mgb2_888k_0.02tsmear_DDB",
    "mgb2_888k_0.04tsmear_DDB",
    "mgb2_121212k_0.01tsmear_DDB",
    #"mgb2_121212k_0.02tsmear_DDB",
    "mgb2_121212k_0.04tsmear_DDB",
]

paths = [os.path.join(abidata.dirpath, "refs", "mgb2_phonons_nkpt_tsmear", f) for f in paths]

robot = abilab.DdbRobot()
for i, path in enumerate(paths):
    robot.add_file(path, path)

robot.remap_labels(lambda ddb: "nkpt: %s, tsmear: %.2f" % (ddb.header["nkpt"], ddb.header["tsmear"]))

r = robot.anaget_phonon_plotters(nqsmall=2)

r.phbands_plotter.gridplot_with_hue("tsmear")

r.phbands_plotter.gridplot_with_hue("tsmear", with_dos=True)

#r.phbands_plotter.gridplot_with_hue("nkpt")

robot.close()



import os
import wx
import collections
import abipy.gui.awx as awx

from monty.collections import AttrDict
from abipy.electrons import ElectronBands


def showElectronDosFrame(parent, filepath):
    """
    Read the electron bands from file filepath and shows an `ElectronDosFrame`.
    """
    bands = ElectronBands.from_file(filepath)
    title = "File: %s" % os.path.relpath(filepath)
    ElectronDosFrame(parent, bands, title=title).Show()


def showElectronBandsPlot(parent, filepath):
    """
    Read the electron bands from file filepath and plot the bands.
    """
    ebands = ElectronBands.from_file(filepath)
    title = "File: %s" % os.path.relpath(filepath)
    ebands.plot(title=title)


def showElectronJdosFrame(parent, filepath):
    """
    Read the electron bands from file filepath and shows an `ElectronJdosFrame`.
    """
    ebands = ElectronBands.from_file(filepath)
    ElectronJdosFrame(parent, ebands).Show()


class DosPanel(awx.Panel):
    """
    Base class defining a panel with controls for specifying the DOS parameters (step, width).
    """
    DEFAULT_WIDTH = 0.2
    DEFAULT_STEP = 0.1

    def __init__(self, parent, **kwargs):
        super(DosPanel, self).__init__(parent, -1, **kwargs)
        self.BuildUi()

    def BuildUi(self):
        main_sizer = wx.BoxSizer(wx.HORIZONTAL)

        label = wx.StaticText(self, -1, "Broadening [eV]:")
        label.Wrap(-1)
        self.width_ctrl = wx.SpinCtrlDouble(self, id=-1, value=str(self.DEFAULT_WIDTH), min=self.DEFAULT_WIDTH/1000, inc=0.1)

        main_sizer.Add(label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)
        main_sizer.Add(self.width_ctrl, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)

        label = wx.StaticText(self, -1, "Mesh step [eV]:")
        label.Wrap(-1)
        self.step_ctrl = wx.SpinCtrlDouble(self, id=-1, value=str(self.DEFAULT_STEP), min=self.DEFAULT_STEP/1000, inc=0.1)

        main_sizer.Add(label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)
        main_sizer.Add(self.step_ctrl, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)

        self.SetSizerAndFit(main_sizer)

    def GetParams(self):
        """Return the parameters for the computation of the DOS in a `AttrDict`."""
        return AttrDict(
            width=float(self.width_ctrl.GetValue()),
            step=float(self.step_ctrl.GetValue()),
        )


class PhononDosPanel(DosPanel):
    """Panel for the specification of the phonon DOS parameters."""
    DEFAULT_WIDTH = 0.002
    DEFAULT_STEP = 0.001


class ElectronDosPanel(DosPanel):
    """Panel for the specification of the electron DOS parameters."""
    DEFAULT_WIDTH = 0.2
    DEFAULT_STEP = 0.1


class ElectronDosDialog(wx.Dialog):
    """Dialog that asks the user to enter the parameters for the electron DOS."""
    def __init__(self, parent, **kwargs):
        if "title" not in kwargs: kwargs["title"] = "Select DOS parameters"
        super(ElectronDosDialog, self).__init__(parent, -1, **kwargs)

        vbox = wx.BoxSizer(wx.VERTICAL)

        self.panel = ElectronDosPanel(self)

        hbox = wx.BoxSizer(wx.HORIZONTAL)

        ok_button = wx.Button(self, wx.ID_OK, label='Ok')
        close_button = wx.Button(self, wx.ID_CANCEL, label='Cancel')

        hbox.Add(ok_button)
        hbox.Add(close_button, flag=wx.LEFT, border=5)

        vbox.Add(self.panel, proportion=0, flag=wx.ALL | wx.EXPAND, border=5)
        vbox.Add(hbox, proportion=0, flag=wx.ALIGN_CENTER | wx.TOP | wx.BOTTOM, border=10)

        self.SetSizerAndFit(vbox)

    def GetParams(self):
        """Return a parameters in a `AttrDict`."""
        return self.panel.GetParams()


class EbandsDosDialog(wx.Dialog):
    """
    Dialog that asks the user to select two files with Ebands and Edos
    and enter the parameters for the electron DOS.
    """
    def __init__(self, parent, filepaths, **kwargs):
        if "title" not in kwargs: kwargs["title"] = "Select paramater forEbands with DOS plot"
        super(EbandsDosDialog, self).__init__(parent, -1, **kwargs)

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        # Use relative paths to save space.
        choices = map(os.path.relpath, filepaths)

        band_label = wx.StaticText(self, -1, "File for bands:"); band_label.Wrap(-1)
        self.bandsfile_cbox = wx.ComboBox(self, id=-1, name='Bands file', choices=choices, value=choices[0], style=wx.CB_READONLY)

        hbox0 = wx.BoxSizer(wx.HORIZONTAL)
        hbox0.Add(band_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)
        hbox0.Add(self.bandsfile_cbox)
        main_sizer.Add(hbox0, proportion=0, flag=wx.ALIGN_CENTER | wx.TOP | wx.BOTTOM, border=10)

        dos_label = wx.StaticText(self, -1, "File for DOS:"); dos_label.Wrap(-1)
        self.dosfile_cbox = wx.ComboBox(self, id=-1, name='Bands file', choices=choices, value=choices[0], style=wx.CB_READONLY)

        hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        hbox1.Add(dos_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)
        hbox1.Add(self.dosfile_cbox)
        main_sizer.Add(hbox1, proportion=0, flag=wx.ALIGN_CENTER | wx.TOP | wx.BOTTOM, border=10)

        self.dos_panel = ElectronDosPanel(self)

        ok_button = wx.Button(self, wx.ID_OK, label='Ok')
        close_button = wx.Button(self, wx.ID_CANCEL, label='Cancel')

        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        hbox2.Add(ok_button)
        hbox2.Add(close_button, flag=wx.LEFT, border=5)

        main_sizer.Add(self.dos_panel, proportion=0, flag=wx.ALL | wx.EXPAND, border=5)
        main_sizer.Add(hbox2, proportion=0, flag=wx.ALIGN_CENTER | wx.TOP | wx.BOTTOM, border=10)

        self.SetSizerAndFit(main_sizer)

    def getEdosParams(self):
        """Return a parameters in a `AttrDict`."""
        return self.dos_panel.GetParams()

    def getBandsDosIndex(self):
        """Return the index of the files for Bands and Dos selected by the user`."""
        ipath = self.bandsfile_cbox.GetSelection()
        imesh = self.dosfile_cbox.GetSelection()
        # Default values if no item is selected.:
        if ipath == wx.NOT_FOUND: ipath = 0
        if imesh == wx.NOT_FOUND: imesh = 0

        return ipath, imesh

class ElectronDosFrame(awx.Frame):
    """
    This frames allows the user to select the parameters and to compute the Electron DOS.
    """
    def __init__(self, parent, bands, **kwargs):
        """
            Args:
                bands:
                    `ElectronBands` instance.
                parent:
                    WX parent window.
        """
        if "title" not in kwargs:
            kwargs["title"] = "Electron DOS"

        if "size" not in kwargs:
            kwargs["size"] = (600, 100)

        super(ElectronDosFrame, self).__init__(parent, id=-1, **kwargs)
        self.bands = bands

        self.BuildUi()

        # Store PlotFrames in this list.
        self._pframes = []

    def BuildUi(self):
        main_sizer = wx.BoxSizer(wx.VERTICAL)

        panel = awx.Panel(self, -1)

        self.dos_panel = ElectronDosPanel(panel)
        main_sizer.Add(self.dos_panel, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        dos_button = wx.Button(panel, -1, "Compute DOS")
        self.Bind(wx.EVT_BUTTON, self.OnClick, dos_button)

        self.replot_checkbox = wx.CheckBox(panel, id=-1, label="Replot")
        self.replot_checkbox.SetValue(True)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        hsizer.Add(dos_button, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
        hsizer.Add(self.replot_checkbox, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)

        main_sizer.Add(hsizer, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)

        panel.SetSizerAndFit(main_sizer)

    def OnClick(self, event):
        p = self.dos_panel.GetParams()
        if p.step == 1234:
            awx.tetris_game()

        try:
            edos = self.bands.get_edos(step=p.step, width=p.width)
        except:
            awx.showErrorMessage(self)
            return

        tot_dos, tot_idos = edos.dos_idos()
        label = "$\sigma = %s, step = %s$" % (p.width, p.step)

        try:
            from wxmplot import PlotApp, PlotFrame
        except ImportError:
            pass

        plotframe = None
        if self.replot_checkbox.GetValue() and len(self._pframes):
            plotframe = self._pframes[-1]

        if plotframe is None:
            plotframe = PlotFrame(parent=self)
            plotframe.plot(tot_dos.mesh, tot_dos.values, label=label, draw_legend=True)
            plotframe.Show()
            self._pframes.append(plotframe)
        else:
            plotframe.oplot(tot_dos.mesh, tot_dos.values, label=label, draw_legend=True)


class ElectronJdosPanel(awx.Panel):
    """
    This panel allows the user to specify the parameters for the
    calculation of the Electron JDOS.
    """
    DEFAULT_STEP = 0.1
    DEFAULT_WIDTH = 0.2

    def __init__(self, parent, nsppol, mband):
        """
            Args:
                parent:
                    WX parent window.
                nsppol:
                    Number of spins.
                mband:
                    Maximum number of bands

        """
        super(ElectronJdosPanel, self).__init__(parent, -1)

        self.nsppol = nsppol
        self.mband = mband

        self.BuildUi()

    def BuildUi(self):
        main_sizer = wx.BoxSizer(wx.VERTICAL)

        sz1 = wx.BoxSizer(wx.HORIZONTAL)

        spin_label = wx.StaticText(self, -1, "Spin:")
        spin_label.Wrap(-1)
        self.spin_cbox = wx.ComboBox(self, id=-1, choices=map(str, range(self.nsppol)), style=wx.CB_READONLY)

        sz1.Add(spin_label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)
        sz1.Add(self.spin_cbox, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        label = wx.StaticText(self, -1, "Broadening [eV]:")
        label.Wrap(-1)
        self.width_ctrl = wx.SpinCtrlDouble(self, id=-1, value=str(self.DEFAULT_WIDTH), min=self.DEFAULT_WIDTH/1000, inc=0.1)

        sz1.Add(label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)
        sz1.Add(self.width_ctrl, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        label = wx.StaticText(self, -1, "Mesh step [eV]:")
        label.Wrap(-1)
        self.step_ctrl = wx.SpinCtrlDouble(self, id=-1, value=str(self.DEFAULT_STEP), min=self.DEFAULT_STEP/1000, inc=0.1)

        sz1.Add(label, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)
        sz1.Add(self.step_ctrl, 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.LEFT, 5)

        main_sizer.Add(sz1)

        start_label = wx.StaticText(self, -1, "Start:")
        start_label.Wrap(-1)
        stop_label = wx.StaticText(self, -1, "Stop:")
        stop_label.Wrap(-1)

        self.vstart = wx.ComboBox(self, id=-1, choices=map(str, range(self.mband)))
        self.vstop = wx.ComboBox(self, id=-1, choices=map(str, range(self.mband)))

        val_sizer = wx.GridSizer(rows=2, cols=2, vgap=5, hgap=5)
        val_sizer.AddMany((start_label, self.vstart))
        val_sizer.AddMany((stop_label, self.vstop))

        val_sb = wx.StaticBox(self, label="Valence bands")
        val_boxsizer = wx.StaticBoxSizer(val_sb, wx.VERTICAL)
        val_boxsizer.Add(val_sizer)

        start_label = wx.StaticText(self, -1, "Start:")
        start_label.Wrap(-1)
        stop_label = wx.StaticText(self, -1, "Stop:")
        stop_label.Wrap(-1)

        self.cstart = wx.ComboBox(self, id=-1, choices=map(str, range(self.mband)))
        self.cstop = wx.ComboBox(self, id=-1, choices=map(str, range(self.mband)))

        cond_sizer = wx.GridSizer(rows=2, cols=2, vgap=5, hgap=5)
        cond_sizer.AddMany((start_label, self.cstart))
        cond_sizer.AddMany((stop_label, self.cstop))

        cond_sb = wx.StaticBox(self, label="Conduction bands")
        cond_boxsizer = wx.StaticBoxSizer(cond_sb, wx.VERTICAL)
        cond_boxsizer.Add(cond_sizer)

        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.AddMany((val_boxsizer, cond_boxsizer))

        main_sizer.Add(hsizer, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 5)
        self.SetSizerAndFit(main_sizer)

    def GetParams(self):
        """Returns the parameter for the JDOS calculation in a `AttrDict`"""
        vstart = int(self.vstart.GetValue())
        vstop = int(self.vstop.GetValue())
        vrange = range(vstart, vstop)
        if vstart == vstop: vrange = vstart

        cstart = int(self.cstart.GetValue())
        cstop = int(self.cstop.GetValue())
        crange = range(cstart, cstop)
        if cstart == cstop: crange = cstart

        return AttrDict(
            vrange=vrange,
            crange=crange,
            spin=int(self.spin_cbox.GetValue()),
            step=float(self.step_ctrl.GetValue()),
            width=float(self.width_ctrl.GetValue()),
            cumulative=True,
        )


class ElectronJdosDialog(wx.Dialog):
    """
    Dialog that asks the user to enter the parameters for the electron JDOS.
    """
    def __init__(self, parent, nsppol, mband, **kwargs):
        """
            Args:
                parent:
                    Parent window.
                nsppol:
                    Number of spins.
                mband
                    Maximum number of bands.
        """
        super(ElectronJdosDialog, self).__init__(parent, -1, **kwargs)

        self.SetSize((250, 200))
        self.SetTitle("Select JDOS parameters")

        vbox = wx.BoxSizer(wx.VERTICAL)

        self.panel = ElectronJdosPanel(self, nsppol, mband)

        hbox = wx.BoxSizer(wx.HORIZONTAL)

        ok_button = wx.Button(self, wx.ID_OK, label='Ok')
        close_button = wx.Button(self, wx.ID_CANCEL, label='Cancel')

        hbox.Add(ok_button)
        hbox.Add(close_button, flag=wx.LEFT, border=5)

        vbox.Add(self.panel, proportion=1, flag=wx.ALL | wx.EXPAND, border=5)
        vbox.Add(hbox, flag=wx.ALIGN_CENTER | wx.TOP | wx.BOTTOM, border=10)

        self.SetSizerAndFit(vbox)

    def GetParams(self):
        """Return a parameters in a `AttrDict`."""
        return self.panel.GetParams()


class ElectronJdosFrame(awx.Frame):
    """
    Frame for the computation of the JDOS.
    """
    def __init__(self, parent, bands, **kwargs):
        """
            Args:
                parent:
                    WX parent window.
                bands:
                    `ElectronBands` instance.
        """
        if "title" not in kwargs:
            kwargs["title"] = "Electron JDOS"

        super(ElectronJdosFrame, self).__init__(parent, id=-1, **kwargs)
        self.bands = bands

        self.BuildUi()

    def BuildUi(self):
        self.jdos_panel = ElectronJdosPanel(self, self.bands.nsppol, self.bands.mband)

        jdos_button = wx.Button(self, -1, "Compute JDOS", (20, 100))
        self.Bind(wx.EVT_BUTTON, self.OnClick, jdos_button)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.jdos_panel, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 5)
        sizer.Add(jdos_button, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 5)

        self.SetSizerAndFit(sizer)

    def OnClick(self, event):
        p = self.jdos_panel.GetParams()
        try:
            self.bands.plot_ejdosvc(vrange=p.vrange, crange=p.crange, step=p.step, width=p.width,
                                   cumulative=p.cumulative)
        except:
            awx.showErrorMessage(self)


class BandsCompareDialog(wx.Dialog):
    """
    Dialog that asks the user to select two files with Ebands and Edos
    and enter the parameters for the electron DOS.
    """
    def __init__(self, parent, filepaths, **kwargs):
        if "title" not in kwargs: kwargs["title"] = "Select files to compare"
        super(BandsCompareDialog, self).__init__(parent, -1, **kwargs)

        main_sizer = wx.BoxSizer(wx.VERTICAL)

        static_sizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, "Files"), wx.VERTICAL)

        self.check_boxes = collections.OrderedDict()

        # Use relative paths to save space.
        self.filepaths = map(os.path.relpath, filepaths)

        for path in self.filepaths:
            assert path not in self.check_boxes
            cbox = wx.CheckBox(self, -1, path, wx.DefaultPosition, wx.DefaultSize, 0)
            cbox.SetValue(True)
            static_sizer.Add(cbox, 0, wx.ALL | wx.EXPAND, 5)
            self.check_boxes[path] = cbox

        main_sizer.Add(static_sizer, 1, wx.EXPAND, 5)

        # Add buttons to (select|deselect) all checkboxes.
        hbox1 = wx.BoxSizer(wx.HORIZONTAL)

        all_button = wx.Button(self, -1, "Select all", wx.DefaultPosition, wx.DefaultSize, 0)
        all_button.Bind(wx.EVT_BUTTON, self.OnSelectAll)
        hbox1.Add(all_button, 0, wx.ALL, 5)

        deselect_button = wx.Button(self, -1, "Deselect all", wx.DefaultPosition, wx.DefaultSize, 0)
        deselect_button.Bind(wx.EVT_BUTTON, self.OnDeselectAll)
        hbox1.Add(deselect_button, 0, wx.ALL, 5)

        main_sizer.Add(hbox1, 0, wx.ALIGN_CENTER_HORIZONTAL, 5)

        ok_button = wx.Button(self, wx.ID_OK, label='Ok')
        close_button = wx.Button(self, wx.ID_CANCEL, label='Cancel')

        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        hbox2.Add(ok_button)
        hbox2.Add(close_button, flag=wx.LEFT, border=5)

        main_sizer.Add(hbox2, proportion=0, flag=wx.ALIGN_CENTER | wx.TOP | wx.BOTTOM, border=10)

        self.SetSizerAndFit(main_sizer)

    def OnSelectAll(self, event):
        for cbox in self.check_boxes.values():
            cbox.SetValue(True)

    def OnDeselectAll(self, event):
        for cbox in self.check_boxes.values():
            cbox.SetValue(False)

    def GetSelectedFilepaths(self):
        """
        Return the list of filepaths selected by the user.
        Main entry point for client code.
        """
        selected = []

        for path, cbox in self.check_boxes.items():
            if cbox.GetValue():
                selected.append(path)

        return selected

    def GetSelectedIndices(self):
        """
        Return the list of selected indices by the user.
        Main entry point for client code.
        """
        selected = []

        for i, cbox in enumerate(self.check_boxes.values()):
            if cbox.GetValue():
                selected.append(i)

        return selected



"""
This module provides objects for extracting timing data from the ABINIT output files
It also provides tools to analyze and to visualize the parallel efficiency.
"""
from abipy.core.mixins import NotebookWriter
from abipy.flowtk import AbinitTimerParser as _Parser


class AbinitTimerParser(_Parser, NotebookWriter):

    def yield_figs(self, **kwargs):  # pragma: no cover
        """
        This function *generates* a predefined list of matplotlib figures with minimal input from the user.
        """
        yield self.plot_stacked_hist(show=False)
        yield self.plot_efficiency(show=False)
        yield self.plot_efficiency(show=False)
        yield self.plot_pie(show=False)

    def write_notebook(self, nbpath=None):
        """
        Write an ipython notebook to nbpath. If nbpath is None, a temporary file in the current
        working directory is created. Return path to the notebook.
        """
        nbformat, nbv, nb = self.get_nbformat_nbv_nb(title=None)
        filenames = self.filenames

        nb.cells.extend([
            nbv.new_code_cell("parser = abilab.AbinitTimerParser()\nparser.parse(%s)" % str(filenames)),
            nbv.new_code_cell("display(parser.summarize())"),
            nbv.new_markdown_cell("# This is a markdown cell"),
            nbv.new_code_cell("parser.plot_stacked_hist();"),
            nbv.new_code_cell("parser.plot_efficiency(what='good', nmax=5);"),
            nbv.new_code_cell("parser.plot_efficiency(what='bad', nmax=5);"),
            nbv.new_markdown_cell("# This is a markdown cell"),
            nbv.new_code_cell("parser.plot_pie();"),

            nbv.new_code_cell("""\
for timer in parser.timers():
    print(timer)
    display(timer.get_dataframe())"""),
        ])

        return self._write_nb_nbpath(nb, nbpath)



"""
Python interface to fftprof. Provides objects to benchmark
the FFT libraries used by ABINIT and plot the results with matplotlib.
"""
import os
import tempfile
import numpy as np

from subprocess import Popen, PIPE
from monty.os.path import which
from monty.fnmatch import WildCard
from abipy.tools.plotting import add_fig_kwargs

__all__ = [
    "FFTBenchmark",
]

_color_fftalg = {
    112: "r",
    #311: "y",
    312: "b",
    412: "y",
    401: "c",
    #511: "c",
    512: "g",
}

_linestyle_nt = {
    1: "-",
    2: "--",
    3: "-.",
    4: ":",
    5: "-",
    6: "--",
    7: "-.",
    8: ":",
}

_markers_nt = {
    1: "1",
    2: ">",
    3: "2",
    4: "<",
    5: "3",
    6: "*",
    7: "4",
    8: "+",
}


class FFT_Test(object):
    """
    This object stores the wall-time of the FFT
    as a function of the size of the problem.
    """
    def __init__(self, ecut, ngfft, wall_time, info):
        """
        Args:
            ecut: List of cutoff energies in Hartree.
            ngfft: List with FFT divisions.
            wall_time: List of wall_time for the different ecut.
            info: Dictionary with extra information.
        """
        self.ecut = np.asarray(ecut)
        self.necut = len(ecut)
        self.ngfft = np.asarray(ngfft, np.int)
        self.wall_time = np.asarray(wall_time)

        self._info = info
        self.__dict__.update(info)
        self.available = int(self.available)

        self.fftalg = int(self.fftalg)
        self.nthreads = int(self.nthreads)

    def __str__(self):
        """String representation."""
        return "fftalg: %s, nt: %d" % (self.fftalg, self.nthreads)

    def speedup_wrt(self, other):
        """Compare self with other, return a new instance of FFT_Test containing the speedup."""
        info = self._info.copy()
        info["speedup"] = other.wall_time / self.wall_time
        return FFT_Test(self.ecut, self.ngfft, self.wall_time, info)

    def plot_ax(self, ax, ydata=None, fact=1.0):
        """Plot ydata on the axis ax. If data is None, the wall_time is plotted."""
        color = _color_fftalg[self.fftalg]
        linestyle = _linestyle_nt[self.nthreads]
        marker = _markers_nt[self.nthreads]

        xx = self.ecut

        if ydata is None:
            yy = self.wall_time
        else:
            yy = self.__dict__[ydata]

        yy = yy * fact

        line, = ax.plot(xx, yy,
                        color=color, linestyle=linestyle, label=str(self), linewidth=3.0,
                        marker=marker, markersize=10)
        return line


class FFTBenchmark(object):
    """
    Container class storing the results of the FFT benchmark.

    Use the class method ``from_file`` to generate a new instance.
    """
    @classmethod
    def from_file(cls, fileobj):
        return parse_prof_file(fileobj)

    def __init__(self, title, FFT_tests):
        self.title = title

        self._fftalgs = []
        for test in FFT_tests:
            alg = test.fftalg
            if alg not in self._fftalgs:
                self._fftalgs.append(alg)

        self._fftalgs.sort()
        self.tests = [t for t in FFT_tests]

    def iter_fftalgs(self):
        """Iterator over the FFT algorithms."""
        return self._fftalgs.__iter__()

    def tests_with_fftalg(self, fftalg):
        """Return the list of FFT_tests with a given fftalg."""
        lst = []
        for t in self.tests:
            if t.fftalg == fftalg: lst.append(t)
        return lst

    @add_fig_kwargs
    def plot(self, exclude_algs=None, exclude_threads=None, **kwargs):
        """
        Plot the wall-time and the speed-up.
        """
        import matplotlib.pyplot as plt
        fig = plt.figure()
        ax1 = fig.add_subplot(2, 1, 1)

        exc_algs = []
        if exclude_algs is not None:
            exc_algs = [int(alg) for alg in exclude_algs]

        exc_nths = []
        if exclude_threads is not None:
            exc_nths = [int(nth) for nth in exclude_threads]

        for test in self.tests:
            if test.fftalg in exc_algs: continue
            if test.available and test.nthreads not in exc_nths:
                #print("test", test.nthreads)
                test.plot_ax(ax1)

        ax1.legend(loc="upper left")
        ax1.grid(True)
        ax1.set_title(self.title)
        ax1.set_xlabel('Ecut [Hartree]')
        ax1.set_ylabel('WALL time (s)')

        ax2 = fig.add_subplot(2, 1, 2)
        ax2.grid(True)

        ax2.set_xlabel('FFT divisions')
        ax2.set_ylabel('Efficiency')

        # Use FFT divs as labels.
        xticks = ax1.get_xticks()
        #print(xticks)

        t0 = self.tests[0]
        labels = [str(ndiv) for ndiv in t0.ngfft]
        #labels = []
        #for t ndiv in zip(t0.ecut, t0.ngfft):
        #for #labels = [ str(ndiv) for ndiv in t0.ngfft ]
        #print labels

        # Rotate labels.
        ax2.set_xticklabels(labels, fontdict=None, minor=False, rotation=35)

        for fftalg in self.iter_fftalgs():
            if fftalg in exc_algs: continue
            tests = self.tests_with_fftalg(fftalg)
            for t in tests:
                if t.nthreads == 1:
                    ref_test = t
                    break
            else:
                raise ValueError("Ref file not found")

            for t in tests:
                tspeed = t.speedup_wrt(ref_test)
                fact = 1.0 / t.nthreads
                if t.nthreads != 1 and t.available and t.nthreads not in exc_nths:
                    tspeed.plot_ax(ax2, ydata="speedup", fact=fact)

        # Use FFT divs as labels.
        xticks = ax1.get_xticks()
        #print(xticks)

        t0 = self.tests[0]
        labels = []
        for xtick in xticks:
            xecut = float(xtick)
            for ecut, ndiv in zip(t0.ecut, t0.ngfft):
                if abs(ecut - xecut) < 0.1:
                    #print(ecut, xecut, ndiv)
                    labels.append(str(ndiv))
                    break
            else:
                msg = "xecut" + str(xecut) + " not found"
                labels.append("")
                #raise RuntimeError(msg)
                #print("labels:", labels)

        # Set and rotate labels.
        ax2.set_xticklabels(labels, fontdict=None, minor=False, rotation=35)

        ideal = [1.0 for i in range(t0.necut)]
        ax2.plot(t0.ecut, ideal, "b-", linewidth=3.0)

        return fig


def parse_prof_file(fileobj):
    """
    Parse the PROF file generated by fftprof.F90.

    Args: fileobj: String or file-like object.

    Returns: Instance of FFTBenchmark.
    """
    # The file contains
    # 1) An initial header with info on the routine.
    # 2) The list of fftalgs that have been analyzed.
    # Results:
    #   ecut ngfft(1:3) wall_times
    #
    # Example

    # Benchmark: routine = fourwf, cplex = 1, option= 2, istwfk= 1
    #  fftalg = 112, fftcache = 16, ndat = 1, nthreads = 1, available = 1
    #  fftalg = 401, fftcache = 16, ndat = 1, nthreads = 1, available = 1
    #  fftalg = 412, fftcache = 16, ndat = 1, nthreads = 1, available = 1
    #  fftalg = 312, fftcache = 16, ndat = 1, nthreads = 1, available = 1
    #   20.0  91  91  90 0.0307 0.0330 0.0275 0.0283
    #   30.0 101 101 100 0.0395 0.0494 0.0404 0.0333

    if not hasattr(fileobj, "readlines"):
        with open(fileobj, "r") as fh:
            lines = fh.readlines()
    else:
        lines = fileobj.readlines()

    # Parse the header
    title = lines.pop(0)[1:]
    line = lines.pop(0)
    info_of_test = []
    while line[0] == "#":
        line = line.replace("#", "")
        line = line.replace("\n", "")
        tokens = line.split(",")
        tokens = [tk.replace(" ", "") for tk in tokens]
        info = {}
        for tk in tokens:
            kw = tk.split("=")
            info[kw[0]] = kw[1]
        info_of_test.append(info)
        line = lines.pop(0)

    # Parse the columns: ecut ngfft(4:6) wall_time[1] wall_time[2] ...
    ntests = len(info_of_test)
    data = [[] for i in range(ntests)]

    ecut, ngfft = [], []
    while line:
        vals = [float(v) for v in line.split()]
        ecut.append(vals[0])
        ngfft.append(vals[1:4])
        for idx, v in enumerate(vals[4:]):
            data[idx].append(v)
        try:
            line = lines.pop(0)
        except IndexError:
            line = None

    # Instantiate FFTBenchmark.
    fft_tests = []
    for idx, wall_time in enumerate(data):
        info = info_of_test[idx]
        Test = FFT_Test(ecut, ngfft, wall_time, info)
        fft_tests.append(Test)

    return FFTBenchmark(title, fft_tests)


class FFTProfError(Exception):
    """Exceptions raised by FFTprof."""


class FFTProf(object):
    """Wrapper around fftprof Fortran executable."""
    Error = FFTProfError

    def __init__(self, fft_input, executable="fftprof"):
        self.verbose = 1
        self.fft_input = fft_input

        self.executable = which(executable)
        if self.executable is None:
            raise self.Error("Cannot find executable %s in $PATH" % executable)

    @classmethod
    def from_file(cls, filename, executable="fftprof"):
        with open(filename, "r") as fh:
            fft_input = fh.read()

        return cls(fft_input, executable=executable)

    def run(self):
        """Execute fftprof in a subprocess."""
        self.workdir = tempfile.mkdtemp()
        print(self.workdir)

        self.stdin_fname = os.path.join(self.workdir, "fftprof.in")
        self.stdout_fname = os.path.join(self.workdir, "fftprof.out")
        self.stderr_fname = os.path.join(self.workdir, "fftprof.err")

        with open(self.stdin_fname, "w") as fh:
            fh.write(self.fft_input)

        args = [self.executable, "<", self.stdin_fname, ">", self.stdout_fname, "2>", self.stderr_fname]

        cmd_str = " ".join(args)

        p = Popen(cmd_str, shell=True, stdout=PIPE, stderr=PIPE, cwd=self.workdir)

        (self.stdout_data, self.stderr_data) = p.communicate()

        self.returncode = p.returncode

        if self.returncode != 0:
            with open(self.stdout_fname, "r") as out, open(self.stderr_fname, "r") as err:
                self.stdout_data = out.read()
                self.stderr_data = err.read()

            if self.verbose:
                print("*** stdout: ***\n", self.stdout_data)
                print("*** stderr: ***\n", self.stderr_data)

            raise self.Error("%s returned %s\n cmd_str: %s" % (self, self.returncode, cmd_str))

        return self.returncode

    def plot(self):
        filepaths = WildCard("PROF_*").filter(os.listdir(self.workdir))
        filepaths = filter(os.path.isfile, [os.path.join(self.workdir, f) for f in filepaths])

        for prof_file in filepaths:
            if self.verbose:
                print("About to plot prof_file: ", prof_file)

            bench = FFTBenchmark.from_file(prof_file)
            bench.plot()



#!/usr/bin/env python
r"""
Effective masses
================

This example shows how to compute and plot the derivatives of the
KS eigenvalues along a high symmetry path in K-space.
"""
from abipy.abilab import abiopen
import abipy.data as abidata

gsr_file = abiopen(abidata.ref_file("si_nscf_GSR.nc"))

kpath = gsr_file.kpoints

#print("ds", kpath.ds)
#for vers in kpath.versors:
#    print("versors", vers)
print(kpath)
#print(kpath.lines)

#branch = bands.get_branch(spin=0, band=3)
#ders = kpath.finite_diff(branch, order=1)
#ders = kpath.finite_diff(branch, order=2)
#print("order2",ders)

ebands = gsr_file.ebands

xys = ebands.derivatives(spin=0, band=0, order=1)
print("xys\n", xys)
#xys = ebands.derivatives(spin=0, band=1, order=1)
#xys = ebands.derivatives(spin=0, band=2, order=1)
#xys = ebands.derivatives(spin=0, band=3, order=1)

#ebands.plot()

emasses = ebands.effective_masses(spin=0, band=0, acc=2)
print("emasses", emasses)

#emasses = ebands.effmass(spin=0, kpoint=[0,0,0], bands=0, acc=2)
#print("emasses", emasses)

#emasses = ebands.effective_masses(spin=0, band=3, acc=2)
#print("emasses", emasses)

#emasses = ebands.effective_masses(spin=0, band=4, acc=2)
#print("emasses", emasses)

gsr_file.close()



""""GUIs for structure."""

import param
import panel as pn
import panel.widgets as pnw
import bokeh.models.widgets as bkw

from abipy.panels.core import AbipyParameterized, gen_id

pn.config.js_files.update({
    '$': 'https://code.jquery.com/jquery-3.4.1.slim.min.js',
    "clipboard": "https://cdn.jsdelivr.net/npm/clipboard@2/dist/clipboard.min.js",
})

pn.config.js_files["ngl"] = "https://cdn.jsdelivr.net/gh/arose/ngl@v2.0.0-dev.33/dist/ngl.js"


_html_with_copy_to_clipboard_ncalls = 0

def html_with_copy_to_clipboard(html, btn_cls="btn-primary btn-sm"):
    global _html_with_copy_to_clipboard_ncalls
    _html_with_copy_to_clipboard_ncalls += 1
    myid = gen_id()
    html = f"""
<button class="btn {btn_cls}" data-clipboard-target="#{myid}"> Copy to clipboard </button>
<div id="{myid}"> {html} </div>
"""
    if _html_with_copy_to_clipboard_ncalls == 1:
        html += "<script>$(document).ready(function () {new ClipboardJS('.btn')})</script>"

    return pn.pane.HTML(html)


class StructurePanel(AbipyParameterized):
    """
    Panel with widgets to interact with an AbiPy Structure
    """
    verbose = param.Integer(0, bounds=(0, None), doc="Verbosity Level")

    # Convert widgets.
    output_format = pnw.Select(name="format", value="abinit",
                               options="abinit,cif,xsf,poscar,qe,siesta,wannier90,cssr,json".split(","))

    # Spglib widgets
    spglib_symprec = pnw.Spinner(name="symprec", value=0.01, start=0.0, end=None, step=0.01)
    spglib_angtol = pnw.Spinner(name="angtol", value=5, start=0.0, end=None, step=1)

    # K-path widgets
    kpath_format = pnw.Select(name="format", value="abinit", options=["abinit", "siesta", "wannier90"])
    line_density = pnw.Spinner(name="line density", value=10, step=5, start=0, end=None)

    # Viewer widgets.
    viewer_btn = pnw.Button(name="View structure", button_type='primary')
    viewer = pnw.Select(name="Viewer", value="vesta",
                        options=["vesta", "xcrysden", "vtk", "matplotlib", "mayavi"])

    # Mp-match
    mp_match_btn = pnw.Button(name="Connect to Materials Project", button_type='primary')

    # Mp-search
    #mp_search_btn = pnw.Button(name="Connect to Materials Project", button_type='primary')
    #mp_api_key

    # GS input generator widgets.
    gs_input_btn = pnw.Button(name="Generate input", button_type='primary')
    gs_type = pnw.Select(name="GS type", value="scf", options=["scf", "relax"])
    kppra = pnw.Spinner(name="kppra", value=1000, step=1000, start=0, end=None)

    label2mode = {
        "unpolarized": 'unpolarized',
        "polarized": 'polarized',
        "anti-ferromagnetic": "afm",
        "non-collinear with magnetism": "spinor",
        "non-collinear, no magnetism": "spinor_nomag",
    }

    spin_mode = pnw.Select(name="SpinMode", value="unpolarized", options=list(label2mode.keys()))

    def __init__(self, structure,  **params):
        super().__init__(**params)
        self.structure = structure

    @param.depends("output_format.value")
    def convert(self):
        return pn.Row(bkw.PreText(text=self.structure.convert(fmt=self.output_format.value)),
                      sizing_mode='stretch_width')

    @param.depends("spglib_symprec.value", "spglib_angtol.value")
    def spglib_summary(self):
        s = self.structure.spget_summary(symprec=self.spglib_symprec.value,
                                         angle_tolerance=self.spglib_angtol.value)
        return pn.Row(bkw.PreText(text=s, sizing_mode='stretch_width'))

    @param.depends("kpath_format.value", "line_density.value")
    def get_kpath(self):
        s = self.structure.get_kpath_input_string(fmt=self.kpath_format.value,
                                                  line_density=self.line_density.value)
        return pn.Row(bkw.PreText(text=s, sizing_mode='stretch_width'))

    @param.depends("viewer_btn.clicks")
    def view(self):
        if self.viewer_btn.clicks == 0: return

        import panel as pn

        #view = self.structure.get_jsmol_view()
        #from ipywidgets_bokeh import IPyWidget
        #view = IPyWidget(widget=view) #, width=800, height=300)
        #from IPython.display import display
        #display(view)
        #return pn.Row(display(view))
        #return pn.panel(view)

        js_files = {'ngl': 'https://cdn.jsdelivr.net/gh/arose/ngl@v2.0.0-dev.33/dist/ngl.js'}
        pn.extension(comms='ipywidgets', js_files=js_files)
        view = self.structure.get_ngl_view()
        return pn.panel(view)

        #pn.config.js_files["ngl"]="https://cdn.jsdelivr.net/gh/arose/ngl@v2.0.0-dev.33/dist/ngl.js"
        #pn.extension()

        html = """<div id="viewport" style="width:100%; height:100%;"></div>
        <script>
        stage = new NGL.Stage("viewport");
        stage.loadFile("rcsb://1NKT.mmtf", {defaultRepresentation: true});
        </script>"""

#        html = """
#         <script>
#    document.addeventlistener("domcontentloaded", function () {
#      var stage = new ngl.stage("viewport");
#      stage.loadfile("rcsb://1crn", {defaultrepresentation: true});
#    });
#  </script>"""

#        html = """
#<script>
#document.addeventlistener("domcontentloaded", function () {
#    // create a `stage` object
#    var stage = new NGL.Stage("viewport");
#    // load a PDB structure and consume the returned `Promise`
#    stage.loadFile("rcsb://1CRN").then(function (component) {
#    // add a "cartoon" representation to the structure component
#    component.addRepresentation("cartoon");
#    // provide a "good" view of the structure
#    component.autoView();
#  });
#});
#</script>"""

        ngl_pane = pn.pane.HTML(html, height=500, width=500)
        return pn.Row(ngl_pane)

        view = self.structure.get_ngl_view()
        #return self.structure.crystaltoolkitview()
        #import nglview as nv
        #view = nv.demo(gui=False)


        #from bokeh.models import ColumnDataSource
        #from bokeh.io import show, curdoc
        #from bokeh.models.widgets import Button, TextInput
        #from bokeh.layouts import layout, widgetbox
        #from jsmol_bokeh_extension import JSMol
        #script_source = ColumnDataSource()

        #info = dict(
        #    height="100%",
        #    width="100%",
        #    serverURL="https://chemapps.stolaf.edu/jmol/jsmol/php/jsmol.php",
        #    use="HTML5",
        #    j2sPath="https://chemapps.stolaf.edu/jmol/jsmol/j2s",
        #    script=
        #    "background black;load https://chemapps.stolaf.edu/jmol/jsmol-2013-09-18/data/caffeine.mol",
        #)

        #applet = JSMol(
        #    width=600,
        #    height=600,
        #    script_source=script_source,
        #    info=info,
        #)

        #button = Button(label='Execute')
        #inp_script = TextInput(value='background white;')

        #def run_script():
        #    script_source.data['script'] = [inp_script.value]

        #button.on_click(run_script)
        #ly = layout([applet, widgetbox(button, inp_script)])
        #show(ly)
        #curdoc().add_root(ly)

        return pn.Row(applet)
        return self.structure.visualize(appname=self.viewer.value)

    @param.depends("gs_input_btn.clicks")
    def on_gs_input_btn(self):
        if self.gs_input_btn.clicks == 0: return
        from abipy.abio.factories import gs_input
        from abipy.data.hgh_pseudos import HGH_TABLE

        gs_inp = gs_input(
            self.structure, HGH_TABLE, kppa=self.kppra.value, ecut=8,
            spin_mode=self.label2mode[self.spin_mode.value],
            smearing=None)
        gs_inp.pop_vars(("charge", "chksymbreak"))
        gs_inp.set_vars(ecut="?? # depends on pseudos", nband="?? # depends on pseudos")

        if self.gs_type.value == "relax":
            gs_inp.set_vars(optcell=2, ionmov=2, ecutsm=0.5, dilatmx=1.05)

        gs_inp.set_mnemonics(False)
        return html_with_copy_to_clipboard(gs_inp._repr_html_())

    @param.depends("mp_match_btn.clicks")
    def on_mp_match_btn(self):
        if self.mp_match_btn.clicks == 0: return
        from abipy.core.structure import mp_match_structure
        mp = mp_match_structure(self.structure, api_key=None, endpoint=None, final=True)
        if not mp.structures:
            raise RuntimeError("No structure found in MP database")

        return pn.Column(self._df(mp.lattice_dataframe), sizing_mode='stretch_width')

    #@param.depends("mp_search_btn.clicks")
    #def on_mp_search_btn(self):
    #    if self.mp_search_btn.clicks == 0: return
    #    from abipy.core.structure import mp_search
    #    chemsys_formula_id = self.stucture.formula
    #    mp = mp_search(chemsys_formula_id, api_key=None, endpoint=None, final=True)
    #    if not mp.structures:
    #        raise RuntimeError("No structure found in MP database")

    #    return pn.Column(self._df(mp.lattice_dataframe), sizing_mode='stretch_width')

    def get_panel(self):
        """Build panel with widgets to interact with the structure either in a notebook or in a bokeh app"""
        tabs = pn.Tabs(); app = tabs.append
        row = pn.Row(bkw.PreText(text=self.structure.to_string(verbose=self.verbose), sizing_mode="scale_both"))
        app(("Summary", row))
        ws = pn.Column('# Spglib options', self.spglib_symprec, self.spglib_angtol)
        app(("Spglib", pn.Row(ws, self.spglib_summary)))
        ws = pn.Column('# K-path options', self.kpath_format, self.line_density)
        app(("Kpath", pn.Row(ws, self.get_kpath)))
        app(("Convert", pn.Row(pn.Column("# Convert structure", self.output_format), self.convert)))
        app(("View", pn.Row(pn.Column("# Visualize structure", self.viewer, self.viewer_btn), self.view)))
        ws = pn.Column('# Generate GS input', self.gs_type, self.spin_mode, self.kppra, self.gs_input_btn)
        app(("GS-input", pn.Row(ws, self.on_gs_input_btn)))
        app(("MP-match", pn.Row(pn.Column(self.mp_match_btn), self.on_mp_match_btn)))

        return tabs



#!/usr/bin/env python
r"""
Unit cell
=========

This example shows how to display the unit cell with matplotlib.
"""
from abipy.abilab import abiopen
import abipy.data as abidata

# Extract structure from netcdf file.
with abiopen(abidata.ref_file("sio2_kpath_GSR.nc")) as gsr:
    structure = gsr.structure

# Visualize sites structure.
structure.plot(color_scheme="Jmol")

# Wrap sites into first unit cell.
# sphinx_gallery_thumbnail_number = 2
structure.plot(to_unit_cell=True)



# coding: utf-8
import os
import tempfile
import warnings

import logging
logger = logging.getLogger(__file__)


def profile(statement, global_vars, local_vars):
    """
    Run statement under profiler, supplying your own globals and locals

    Example::

        stats = profile("main()", global_vars=globals(), local_vars=locals())
    """
    import pstats
    import cProfile
    _, filename = tempfile.mkstemp()
    cProfile.runctx(statement, global_vars, local_vars, filename=filename)

    s = pstats.Stats(filename)
    s.strip_dirs().sort_stats("time")
    s.print_stats()
    os.remove(filename)
    return s


class HtmlDiff(object):
    """
    This object produces diff files in HTML format and displays them in the browser.

    Usage example:

    .. code-block:: python

        HtmlDiff(filepaths).open_browser()
    """
    def __init__(self, filepaths):
        if len(filepaths) < 2:
            raise ValueError("You need more than one file to compare!")
        self.filepaths = filepaths

    def open_browser(self, diffmode="difflib", **kwargs):
        """
        Generate diff with ``diffmode``, open browser, return exit code.
        """
        try:
            func = getattr(self, diffmode)
        except AttributeError:
            raise AttributeError("Unsupported value for diffmode: `%s`" % str(diffmode))

        return func(**kwargs)

    def _launch_browser(self, tmpname):  # pragma: no cover
        """Open ``tmpname`` file in the default browser."""
        # warning: This code is not portable since we should pass a url.
        if not tmpname.startswith("file://"): tmpname = "file://" + tmpname
        import webbrowser
        try:
            return int(webbrowser.open(tmpname))
        except webbrowser.Error as exc:
            # Warn the user and ignore the exception.
            warnings.warn(str(exc))
            return 1

    def difflib(self, **kwargs):
        """
        Use difflib to generate a HTML file with the diff.
        Open file in the browser.
        """
        with open(self.filepaths[0], 'rt') as fh:
            fromlines = fh.readlines()

        diffs = []
        for path in self.filepaths[1:]:
            with open(path, 'rt') as fh:
                tolines = fh.readlines()

            _, tmpname = tempfile.mkstemp(suffix=".html", text=True)
            import difflib
            #diff = difflib.HtmlDiff().make_table(fromlines, tolines,
            diff = difflib.HtmlDiff().make_file(fromlines, tolines,
                                                self.filepaths[0], path) #context=options.c, numlines=n)
            with open(tmpname, "wt") as fh:
                fh.writelines(diff)

            return self._launch_browser(tmpname)

    def pygmentize(self):
        """
        Execute ``diff`` and ``pygmentize`` in a subprocess to generate a HTML file with the diff.
        Open file in the browser.
        """
        for file2 in self.filepaths[1:]:
            _, tmpname = tempfile.mkstemp(suffix=".html", text=True)

            # https://stackoverflow.com/questions/641055/diff-to-html-diff2html-program
            #cmd = "/usr/bin/diff -y %s %s | pygmentize -l diff -f html -O full -o %s" % (
            #    self.filepaths[0], file2, tmpname)
            cmd = "diff -U9999999 %s %s | pygmentize -l diff -f html -O full -o %s" % (
                self.filepaths[0], file2, tmpname)

            retcode = os.system(cmd)
            if retcode != 0:
                print("Non-zero exit status returned by: `%s", cmd)
                print("Please install pygmentize with `pip install pygmentize` or `conda install pygmentize`")
                return retcode

            return self._launch_browser(tmpname)



#!/usr/bin/env python
r"""
Phonon Bands with/without ASR
=============================

This example shows how to plot a phonon band structure
with and without imposing the acoustic sum rule at the Gamma point.
"""
from abipy import abilab
import abipy.data as abidata

# Open DDB file
filepath = abidata.ref_file("mp-1009129-9x9x10q_ebecs_DDB")
ddb = abilab.abiopen(filepath)

# This method computes the phonon bands and DOS by calling anaddb
# with different values of asr and returns a PhononBandsPlotter object.
plotter = ddb.anacompare_asr(asr_list=(0, 2))
plotter.combiplot()

# Set nqsmall to 0 to disable DOS computation.
plotter = ddb.anacompare_asr(asr_list=(0, 2), nqsmall=0, ndivsm=10)
plotter.gridplot()

ddb.close()
